/* Adapted from earth.c and examples in OpenGL redbook chapter 2  */


#include <iostream>
using namespace std;


#include <cstdlib>
#include <GL/glut.h>
#include <cmath>

long depth = 0;

void normalize(float v[3]) {    
   GLfloat d = sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2]); 
   if (d == 0.0) {
      cout << "zero length vector" << endl;    
      return;
   }
   v[0] /= d; v[1] /= d; v[2] /= d; 
}


void drawtriangle(float *v1, float *v2, float *v3) 
{ 
   float color[3];

   for (int i = 0; i < 3; i++)  
      color[i] = v1[i]+v2[i]+v3[i]+0.1; 

   normalize(color);
   
   glColor3f(color[0], color[1], color[2]);

   glBegin(GL_POLYGON); 

	  glVertex3fv(v1); 
 
	  glVertex3fv(v2);  

	  glVertex3fv(v3);    
   glEnd(); 
}


void subdivide(float *v1, float *v2, float *v3, long depth)
{
   GLfloat v12[3], v23[3], v31[3];
   GLint i;

   if (depth == 0) {
      drawtriangle(v1, v2, v3);
      return;
   }
   for (i = 0; i < 3; i++) {
      v12[i] = v1[i]+v2[i];
      v23[i] = v2[i]+v3[i];
      v31[i] = v3[i]+v1[i];
   }
   normalize(v12);
   normalize(v23);
   normalize(v31);
   subdivide(v1, v12, v31, depth-1);
   subdivide(v2, v23, v12, depth-1);
   subdivide(v3, v31, v23, depth-1);
   subdivide(v12, v23, v31, depth-1);
}

void subdivide(float *v1, float *v2, float *v3) 
{ 
   GLfloat v12[3], v23[3], v31[3];    
   GLint i;

   for (i = 0; i < 3; i++) { 
      v12[i] = v1[i]+v2[i]; 
      v23[i] = v2[i]+v3[i];     
      v31[i] = v3[i]+v1[i];    
   } 
   normalize(v12);    
   normalize(v23); 
   normalize(v31); 
   drawtriangle(v1, v12, v31);    
   drawtriangle(v2, v23, v12);    
   drawtriangle(v3, v31, v23);    
   drawtriangle(v12, v23, v31); 
}


	// Geometric information abour icosahedron 

	#define X .525731112119133606 
	#define Z .850650808352039932 

	static GLfloat vdata[12][3] = {    
	   {-X, 0.0, Z}, {X, 0.0, Z}, {-X, 0.0, -Z}, {X, 0.0, -Z},    
	   {0.0, Z, X}, {0.0, Z, -X}, {0.0, -Z, X}, {0.0, -Z, -X},    
	   {Z, X, 0.0}, {-Z, X, 0.0}, {Z, -X, 0.0}, {-Z, -X, 0.0} 
	};

	static GLint tindices[20][3] = { 
	   {0,4,1}, {0,9,4}, {9,5,4}, {4,5,8}, {4,8,1},    
	   {8,10,1}, {8,3,10}, {5,3,8}, {5,2,3}, {2,7,3},    
	   {7,10,3}, {7,6,10}, {7,11,6}, {11,0,6}, {0,1,6}, 
	   {6,1,10}, {9,0,11}, {9,11,2}, {9,2,5}, {7,2,11} };


void icosahedron( )
{
	/* clear window */
	glClear(GL_COLOR_BUFFER_BIT);



	//**********************************************************************************************************
	for (int i = 0; i < 20; i++) 
	{    
			subdivide(	&vdata[tindices[i][0]][0], 
						&vdata[tindices[i][1]][0], 
						&vdata[tindices[i][2]][0],
						depth);
	}
	//**********************************************************************************************************



	glFlush();

}


//*************************************************************************
//	Adpated from earth.cpp below
//*************************************************************************

static GLfloat theta[] = {0.0,0.0,0.0};
static GLint axis = 2;


void myDisplay(void)
{
 /* display callback, clear frame buffer and z buffer,
   rotate cube and draw, swap buffers */

 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glRotatef(theta[0], 1.0, 0.0, 0.0);
	glRotatef(theta[1], 0.0, 1.0, 0.0);
	glRotatef(theta[2], 0.0, 0.0, 1.0);

 icosahedron();

 glFlush();
 glutSwapBuffers();
}


void myMouse(int btn, int state, int x, int y)
{

/* mouse callback, selects an axis about which to rotate */

	if(btn==GLUT_LEFT_BUTTON && state == GLUT_DOWN)
		depth++;
	else if(btn==GLUT_RIGHT_BUTTON && state == GLUT_DOWN) 
	{
		if (depth > 0)
			depth--;
	}

	glutPostRedisplay();
}

void myKeyboard(unsigned char c, int x, int y)
{
	/* keyboard callback, selects an axis about which to rotate */

	if(c =='1') theta[0]+=2;
	if(c =='2') theta[1]+=2;
	if(c =='3') theta[2]+=2;
	glutPostRedisplay();
}

void myReshape(int w, int h)
{
    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    if (w <= h)
        glOrtho(-2.0, 2.0, -2.0 * (GLfloat) h / (GLfloat) w,
            2.0 * (GLfloat) h / (GLfloat) w, -10.0, 10.0);
    else
        glOrtho(-2.0 * (GLfloat) w / (GLfloat) h,
            2.0 * (GLfloat) w / (GLfloat) h, -2.0, 2.0, -10.0, 10.0);
    glMatrixMode(GL_MODELVIEW);
}

void main(int argc, char **argv)
{
	cout << "*********************************************************" << endl;
	cout << "Click left button to increase the depth of approximation " << endl;
 	cout << "Click rightt button to decrease the depth of approximation " << endl;
 	cout << "Press 1 or 2 or 3 to rotate in different axes " << endl;
	cout << "*********************************************************" << endl;
   glutInit(&argc, argv);

    //glutInitDisplayMode(GLUT_RGB );				//RGB, & default single buffering 
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);	//RGB, & double buffering 

    glutInitWindowSize(500, 500);
    glutCreateWindow("sphere");

    glutReshapeFunc(myReshape);
    glutDisplayFunc(myDisplay);
	glutKeyboardFunc(myKeyboard);
	glutMouseFunc(myMouse);

	glClearColor(1.0, 1.0, 1.0, 1.0);
	glColor3f(0.0, 0.0, 0.0);
	glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);	
	//Both faces shown if visible, draw only the edges of polygons

    glutMainLoop();
}
